@@ -426,10 +426,8 @@ osmo_sccp_simple_client_on_ss7_id(void *ctx, uint32_t ss7_id, const char *name,
 			goto out_rt;
 		asp_created = true;
 
-		asp->cfg.local.host =
-		    default_local_ip ? asp->cfg.local.host : NULL;
-		asp->cfg.remote.host =
-		    default_remote_ip ? asp->cfg.remote.host : NULL;
+		asp->cfg.local.host = default_local_ip ? talloc_strdup(asp, default_local_ip) : NULL;
+		asp->cfg.remote.host = default_remote_ip ? talloc_strdup(asp, default_remote_ip) : NULL;
 
 		osmo_ss7_as_add_asp(as, asp->cfg.name);
 	}
